@@ -22,7 +22,6 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
-#include <mntent.h>
 #include <sys/vfs.h>
 
 #include "automount.h"
@@ -56,7 +55,7 @@ static int dev_ioctl_send_ready(unsigned int, int, unsigned int);
 static int dev_ioctl_send_fail(unsigned int, int, unsigned int, int);
 static int dev_ioctl_setpipefd(unsigned int, int, int);
 static int dev_ioctl_catatonic(unsigned int, int);
-static int dev_ioctl_timeout(unsigned int, int, time_t *);
+static int dev_ioctl_timeout(unsigned int, int, time_t);
 static int dev_ioctl_requestor(unsigned int, int, const char *, uid_t *, gid_t *);
 static int dev_ioctl_expire(unsigned int, int, const char *, unsigned int);
 static int dev_ioctl_askumount(unsigned int, int, unsigned int *);
@@ -70,7 +69,7 @@ static int ioctl_close(unsigned int, int);
 static int ioctl_send_ready(unsigned int, int, unsigned int);
 static int ioctl_send_fail(unsigned int, int, unsigned int, int);
 static int ioctl_catatonic(unsigned int, int);
-static int ioctl_timeout(unsigned int, int, time_t *);
+static int ioctl_timeout(unsigned int, int, time_t);
 static int ioctl_expire(unsigned int, int, const char *, unsigned int);
 static int ioctl_askumount(unsigned int, int, unsigned int *);
 
@@ -121,17 +120,12 @@ void init_ioctl_ctl(void)
 	if (ctl.ops)
 		return;
 
-	devfd = open(CONTROL_DEVICE, O_RDONLY);
+	devfd = open_fd(CONTROL_DEVICE, O_RDONLY);
 	if (devfd == -1)
 		ctl.ops = &ioctl_ops;
 	else {
 		struct autofs_dev_ioctl param;
 
-		int cl_flags = fcntl(devfd, F_GETFD, 0);
-		if (cl_flags != -1) {
-			cl_flags |= FD_CLOEXEC;
-			fcntl(devfd, F_SETFD, cl_flags);
-		}
 		/*
 		 * Check compile version against kernel.
 		 * Selinux may allow us to open the device but not
@@ -276,8 +270,10 @@ static struct autofs_dev_ioctl *alloc_dev_ioctl_path(int ioctlfd, const char *pa
 	struct autofs_dev_ioctl *ioctl;
 	size_t size, p_len;
 
-	if (!path)
+	if (!path) {
+		errno = EINVAL;
 		return NULL;
+	}
 
 	p_len = strlen(path);
 	size = sizeof(struct autofs_dev_ioctl) + p_len + 1;
@@ -378,20 +374,14 @@ static int ioctl_open(unsigned int logopt,
 		      int *ioctlfd, dev_t devid, const char *path)
 {
 	struct statfs sfs;
-	int save_errno, fd, cl_flags;
+	int save_errno, fd;
 
 	*ioctlfd = -1;
 
-	fd = open(path, O_RDONLY);
+	fd = open_fd(path, O_RDONLY);
 	if (fd == -1)
 		return -1;
 
-	cl_flags = fcntl(fd, F_GETFD, 0);
-	if (cl_flags != -1) {
-		cl_flags |= FD_CLOEXEC;
-		fcntl(fd, F_SETFD, cl_flags);
-	}
-
 	if (fstatfs(fd, &sfs) == -1) {
 		save_errno = errno;
 		goto err;
@@ -589,25 +579,24 @@ static int ioctl_catatonic(unsigned int logopt, int ioctlfd)
 }
 
 /* Set the autofs mount timeout */
-static int dev_ioctl_timeout(unsigned int logopt, int ioctlfd, time_t *timeout)
+static int dev_ioctl_timeout(unsigned int logopt, int ioctlfd, time_t timeout)
 {
 	struct autofs_dev_ioctl param;
 
 	init_autofs_dev_ioctl(&param);
 	param.ioctlfd = ioctlfd;
-	param.timeout.timeout = *timeout;
+	param.timeout.timeout = timeout;
 
 	if (ioctl(ctl.devfd, AUTOFS_DEV_IOCTL_TIMEOUT, &param) == -1)
 		return -1;
 
-	*timeout = param.timeout.timeout;
-
 	return 0;
 }
 
-static int ioctl_timeout(unsigned int logopt, int ioctlfd, time_t *timeout)
+static int ioctl_timeout(unsigned int logopt, int ioctlfd, time_t timeout)
 {
-	return ioctl(ioctlfd, AUTOFS_IOC_SETTIMEOUT, timeout);
+	time_t tout = timeout;
+	return ioctl(ioctlfd, AUTOFS_IOC_SETTIMEOUT, &tout);
 }
 
 /*
@@ -776,12 +765,10 @@ static int dev_ioctl_ismountpoint(unsigned int logopt,
 	if (err) {
 		*mountpoint = DEV_IOCTL_IS_MOUNTED;
 
-		if (param->ismountpoint.out.magic) {
-			if (param->ismountpoint.out.magic == AUTOFS_SUPER_MAGIC)
-				*mountpoint |= DEV_IOCTL_IS_AUTOFS;
-			else
-				*mountpoint |= DEV_IOCTL_IS_OTHER;
-		}
+		if (param->ismountpoint.out.magic == AUTOFS_SUPER_MAGIC)
+			*mountpoint |= DEV_IOCTL_IS_AUTOFS;
+		else
+			*mountpoint |= DEV_IOCTL_IS_OTHER;
 	}
 
 	free_dev_ioctl_path(param);
